local iftracker = CreateFrame("Frame","IFTrackerContainer",UIParent)
iftracker:SetFrameStrata("BACKGROUND")
iftracker:SetAlpha(0.5)
iftracker:SetMinResize(32, 32)

iftracker.BELOW_THREE_MSG = "CHILL"
iftracker.ABOVE_THREE_MSG = "ENGAGE"
iftracker.IDLE_MSG = "IDLE"
iftracker.IDLE_VALUE_MSG = "0"

iftracker:RegisterEvent("ADDON_LOADED");
iftracker:RegisterEvent("PLAYER_ENTERING_WORLD");
iftracker:RegisterEvent("PLAYER_TALENT_UPDATE");
iftracker:SetScript("OnEvent", function(self,event,name)
	if event == "ADDON_LOADED" and name == "IFTracker" then
		if IFTrackerSettings == nil then
			IFTrackerSettings = {}
			IFTrackerSettings.point = "CENTER"
			IFTrackerSettings.xOfs = 0
			IFTrackerSettings.yOfs = 0
			IFTrackerSettings.width = 128
			IFTrackerSettings.height = 192
		end
		self:SetWidth(IFTrackerSettings.width)
		self:SetHeight(IFTrackerSettings.height)
		self:SetPoint(IFTrackerSettings.point,IFTrackerSettings.xOfs,IFTrackerSettings.yOfs)				
		self:CreateFrames()
	elseif event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_TALENT_UPDATE" then
		local group = GetActiveSpecGroup()
		local _,name,_,selected = GetTalentInfo( 6, 3, group )
		if name == "Incanter's Flow" and selected == true then
			self:Show()
		else
			self:Hide()
		end
	end
end)

function iftracker:CreateFrames()
	local texture = self:CreateTexture(nil,"BACKGROUND")
	texture:SetTexture(0,0,0,.25)
	texture:SetAllPoints(iftracker)
	texture.texture = texture

	self.statusFrame = CreateFrame("Frame","IFTrackerStatusTextFrame",self)

	local status_texture = self.statusFrame:CreateTexture(nil,"BACKGROUND")
	status_texture:SetTexture(0,0,0,0.25)
	status_texture:SetAllPoints(self.statusFrame)
	self.statusFrame.texture = status_texture

	local status_text = self.statusFrame:CreateFontString(nil, "OVERLAY", "GameFontRed")
	status_text:SetJustifyH("Left")
	status_text:SetText(self.IDLE_VALUE_MSG)
	status_text:SetTextColor(0.5, 0.5, 0.5, 1)
	self.statusText = status_text

	self.iconFrame = CreateFrame("Frame","IFTrackerIconFrame",iftracker)
	self.iconFrame:SetResizable(true)

	local icon_texture = self.iconFrame:CreateTexture(nil,"BACKGROUND")
	icon_texture:SetTexture("Interface\\Icons\\Ability_Mage_IncantersAbsorbtion.blp")
	icon_texture:SetAllPoints(self.iconFrame)
	self.iconFrame.texture = icon_texture

	self.messageFrame = CreateFrame("Frame","IFTrackerMessageTextFrame",self.iconFrame)

	local message_texture = self.messageFrame:CreateTexture(nil,"BACKGROUND")
	message_texture:SetTexture(0,0,0,0.5)
	message_texture:SetAllPoints(self.messageFrame)
	self.messageFrame.texture = message_texture

	self.messageText = self.messageFrame:CreateFontString(nil, "OVERLAY", "GameFontRed")
	self.messageText:SetJustifyH("Left")
	self.messageText:SetText(self.IDLE_MSG)
	self.messageText:SetTextColor(0.5, 0.5, 0.5, 1)

	self:SizeAndPositionFrames()
	self:registerEvents()
end

function iftracker:SizeAndPositionFrames()
	self.statusFrame:SetWidth(self:GetWidth())
	self.statusFrame:SetHeight(self:GetHeight()/3)
	self.statusFrame:SetPoint("TOP",0,0)
	self.statusText:SetFont("Fonts\\FRIZQT__.TTF", self:GetHeight()/3)
	self.statusText:SetHeight(self:GetHeight()/3)
	self.statusText:SetPoint("CENTER",self.statusFrame ,0,0)
	self.iconFrame:SetWidth(self:GetWidth())
	self.iconFrame:SetHeight(self:GetWidth())
	self.iconFrame:SetPoint("TOP",0,-(self.statusFrame:GetHeight()))
	self.messageFrame:SetWidth(self:GetWidth())
	self.messageFrame:SetHeight(self:GetHeight()/3)
	self.messageFrame:SetPoint("CENTER",0,0)
	self.messageText:SetFont("Fonts\\FRIZQT__.TTF", self:GetHeight()/3)
	self.messageText:SetHeight(self:GetHeight()/3)
	self.messageText:SetPoint("CENTER",self.messageFrame ,0,0)
end

function iftracker:registerEvents()
	self:RegisterForDrag("LeftButton","RightButton")
	self:SetScript("OnDragStart", function(self,event)
		if event == "LeftButton" then
			self:StartMoving()
		else
			self:SetScript("OnUpdate", function(self,elapsed)
				self:SizeAndPositionFrames()
				if self.isActive then
					self.lastUpdateTime = self.lastUpdateTime + elapsed
					if (self.lastUpdateTime > self.updateInterval) then
						local name,_,_,count= UnitBuff("player", "Incanter's Flow");
						if name and count then
							local delta = count - self.lastStack
							self.lastStack = count
							self.statusText:SetText(count)
							if delta < 0 and count <= 3 then
								self.statusText:SetTextColor(1, 0, 0, 1)
								self.messageText:SetTextColor(1, 0, 0, 1)
								self.messageText:SetText(self.BELOW_THREE_MSG)
							elseif delta > 0  and count >= 3 then
								self.statusText:SetTextColor(0, 1, 0, 1)
								self.messageText:SetTextColor(0, 1, 0, 1)
								self.messageText:SetText(self.ABOVE_THREE_MSG)
							end
							self.lastUpdateTime = 0;
						end
					end
				end
			end)
			self:StartSizing()
		end
	end)
	self:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		IFTrackerSettings.top = self:GetTop()
		IFTrackerSettings.left = self:GetLeft()
		local point, _, _, xOfs, yOfs = self:GetPoint()
		IFTrackerSettings.point = point
		IFTrackerSettings.xOfs = xOfs
		IFTrackerSettings.yOfs = yOfs
		IFTrackerSettings.width = self:GetWidth()
		IFTrackerSettings.height = self:GetHeight()
	end)

	self.isActive = false
	self.updateInterval = 1.0
	self.lastUpdateTime = 0
	self.lastStack = 1
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")

	self:SetScript("OnEvent", function(self,event)
		if event == "PLAYER_REGEN_DISABLED" then
			self.isActive = true
			self.messageText:SetTextColor(1, 0, 0, 1)
			self.statusText:SetTextColor(1, 0, 0, 1)
			self:SetAlpha(1.0)
		else
			self.isActive = false
			self.messageText:SetText(self.IDLE_MSG)
			self.statusText:SetText(self.IDLE_VALUE_MSG)
			self.messageText:SetTextColor(0.5, 0.5, 0.5, 1)
			self.statusText:SetTextColor(0.5, 0.5, 0.5, 1)
			self:SetAlpha(0.5)
		end
	end)

	SLASH_IFTracker1, SLASH_IFTracker2,SLASH_IFTracker3 = "/ift", "/ift lock","/ift unlock";
	function SlashCmdList.IFTracker(msg)
		if msg == "lock" then
	    	self:SetMovable(false)
			self:EnableMouse(false)
			self:SetResizable(false)
	    elseif msg == "unlock" then
	    	self:SetMovable(true)
			self:EnableMouse(true)
			self:SetResizable(true)
		else
			print("|cFF3399FFIFTracker commands|r");
			print("|cFF3399FF/ift|r lock [left click drag to move. right click drag to size.]")
			print("|cFF3399FF/ift|r unlock ")
	    end 
	end
end